# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from pkg_resources import resource_stream  # @UnresolvedImport
from . import ReverseConverter
from ..exceptions import ConvertError, ReverseError


class Alpha3BConverter(ReverseConverter):
    def __init__(self):
        self.codes = set()
        self.to_alpha3b = {}
        self.from_alpha3b = {}
        with resource_stream('babelfish', 'data/iso-639-3.tab') as f:
            f.readline()
            for l in f:
                (alpha3, alpha3b, _, _, _, _, _, _) = l.decode('utf-8').split('\t')
                if alpha3b != '':
                    self.codes.add(alpha3b)
                    self.to_alpha3b[alpha3] = alpha3b
                    self.from_alpha3b[alpha3b] = alpha3

    def convert(self, alpha3, country=None):
        if alpha3 not in self.to_alpha3b:
            raise ConvertError(alpha3, country)
        return self.to_alpha3b[alpha3]

    def reverse(self, alpha3b):
        if alpha3b not in self.from_alpha3b:
            raise ReverseError(alpha3b)
        return (self.from_alpha3b[alpha3b], None)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from pkg_resources import resource_stream  # @UnresolvedImport
from . import ReverseConverter
from ..exceptions import ConvertError, ReverseError


class Alpha2Converter(ReverseConverter):
    def __init__(self):
        self.codes = set()
        self.to_alpha2 = {}
        self.from_alpha2 = {}
        with resource_stream('babelfish', 'data/iso-639-3.tab') as f:
            f.readline()
            for l in f:
                (alpha3, _, _, alpha2, _, _, _, _) = l.decode('utf-8').split('\t')
                if alpha2 != '':
                    self.codes.add(alpha2)
                    self.to_alpha2[alpha3] = alpha2
                    self.from_alpha2[alpha2] = alpha3

    def convert(self, alpha3, country=None):
        if alpha3 not in self.to_alpha2:
            raise ConvertError(alpha3, country)
        return self.to_alpha2[alpha3]

    def reverse(self, alpha2):
        if alpha2 not in self.from_alpha2:
            raise ReverseError(alpha2)
        return (self.from_alpha2[alpha2], None)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from .alpha3b import Alpha3BConverter


class OpenSubtitlesConverter(Alpha3BConverter):
    def __init__(self):
        super(OpenSubtitlesConverter, self).__init__()
        self.codes.add('pob')
        self.to_opensubtitles = {('por', 'BR'): 'pob'}
        self.from_opensubtitles = {'pob': ('por', 'BR')}

    def convert(self, alpha3, country=None):
        alpha3b = super(OpenSubtitlesConverter, self).convert(alpha3, country)
        if (alpha3b, country) in self.to_opensubtitles:
            return self.to_opensubtitles[(alpha3b, country)]
        return alpha3b

    def reverse(self, opensubtitles):
        if opensubtitles in self.from_opensubtitles:
            return self.from_opensubtitles[opensubtitles]
        return super(OpenSubtitlesConverter, self).reverse(opensubtitles)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from pkg_resources import resource_stream  # @UnresolvedImport
from . import ReverseConverter
from ..exceptions import ConvertError, ReverseError


class NameConverter(ReverseConverter):
    def __init__(self):
        self.codes = set()
        self.to_name = {}
        self.from_name = {}
        with resource_stream('babelfish', 'data/iso-639-3.tab') as f:
            f.readline()
            for l in f:
                (alpha3, _, _, _, _, _, name, _) = l.decode('utf-8').split('\t')
                self.codes.add(name)
                self.to_name[alpha3] = name
                self.from_name[name] = alpha3

    def convert(self, alpha3, country=None):
        if alpha3 not in self.to_name:
            raise ConvertError(alpha3, country)
        return self.to_name[alpha3]

    def reverse(self, name):
        if name not in self.from_name:
            raise ReverseError(name)
        return (self.from_name[name], None)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals


class Error(Exception):
    """Base class for all exceptions in babelfish"""
    pass


class ConvertError(Error):
    """Exception raised by converters when :meth:`~babelfish.converters.Converter.convert` fails

    :param string alpha3: alpha3 code that failed conversion
    :param country: country code that failed conversion, if any
    :type country: string or None

    """
    def __init__(self, alpha3, country):
        self.alpha3 = alpha3
        self.country = country

    def __str__(self):
        if self.country is None:
            return repr(self.alpha3)
        return repr(self.alpha3 + '-' + self.country)


class ReverseError(Error):
    """Exception raised by converters when :meth:`~babelfish.converters.ReverseConverter.reverse` fails

    :param string code: code that failed reverse conversion

    """
    def __init__(self, code):
        self.code = code

    def __str__(self):
        return repr(self.code)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from pkg_resources import resource_stream  # @UnresolvedImport


COUNTRIES = {}
with resource_stream('babelfish', 'data/iso-3166-1.txt') as f:
    f.readline()
    for l in f:
        (name, alpha2) = l.decode('utf-8').strip().split(';')
        COUNTRIES[alpha2] = name


class Country(object):
    """A country on Earth

    A country is represented by a two-letters code from the ISO-3166 standard

    :param string country: two-letters ISO-3166 country code

    """
    def __init__(self, country):
        if country not in COUNTRIES:
            raise ValueError('%r is not a valid country' % country)

        #: ISO-3166 two-letters country code
        self.alpha2 = country

    @property
    def name(self):
        """English name of the country"""
        return COUNTRIES[self.alpha2]

    def __hash__(self):
        return hash(self.alpha2)

    def __eq__(self, other):
        return self.alpha2 == other.alpha2

    def __ne__(self, other):
        return not self == other

    def __repr__(self):
        return '<Country [%s]>' % self

    def __str__(self):
        return self.alpha2



# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#


class Converter(object):
    """A :class:`Converter` supports converting an alpha3 language code with an
    alpha2 country code into a custom code

    .. attribute:: codes

        Set of possible custom codes

    """
    def convert(self, alpha3, country=None):
        """Convert an alpha3 language code with an alpha2 country code
        into a custom code

        :param string alpha3: ISO-639-3 language code
        :param country: ISO-3166 country code, if any
        :type country: string or None
        :return: the corresponding custom code
        :rtype: string
        :raise: :class:`~babelfish.exceptions.ConvertError`

        """
        raise NotImplementedError


class ReverseConverter(Converter):
    """A :class:`Converter` able to reverse a custom code into a alpha3
    ISO-639-3 language code and alpha2 ISO-3166-1 country code

    """
    def reverse(self, code):
        """Reverse a custom code into alpha3 and country code

        :param string code: custom code to reverse
        :return: the corresponding alpha3 ISO-639-3 language code and alpha2 ISO-3166-1 country code
        :rtype: tuple
        :raise: :class:`~babelfish.exceptions.ReverseError`

        """
        raise NotImplementedError



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
__title__ = 'babelfish'
__version__ = '0.2.0'
__author__ = 'Antoine Bertin'
__license__ = 'BSD'
__copyright__ = 'Copyright 2013 the BabelFish authors'

from .converters import Converter, ReverseConverter
from .country import COUNTRIES, Country
from .exceptions import Error, ConvertError, ReverseError
from .language import (CONVERTERS, LANGUAGES, Language, register_converter, unregister_converter, load_converters,
    clear_converters)


load_converters()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from unittest import TestCase, TestSuite, TestLoader, TextTestRunner
from babelfish import (LANGUAGES, Language, Country, CONVERTERS, ReverseConverter, load_converters, clear_converters,
    register_converter, unregister_converter, ConvertError, ReverseError)


class TestCountry(TestCase):
    def test_wrong_country(self):
        with self.assertRaises(ValueError):
            Country('ZZ')

    def test_eq(self):
        self.assertTrue(Country('US') == Country('US'))

    def test_ne(self):
        self.assertTrue(Country('GB') != Country('US'))

    def test_hash(self):
        self.assertTrue(hash(Country('US')) == hash('US'))


class TestLanguage(TestCase):
    def test_languages(self):
        self.assertTrue(len(LANGUAGES) == 7874)

    def test_wrong_language(self):
        with self.assertRaises(ValueError):
            Language('zzz')

    def test_converter_alpha2(self):
        self.assertTrue(Language('eng').alpha2 == 'en')
        self.assertTrue(Language.fromalpha2('en') == Language('eng'))
        self.assertTrue(Language.fromcode('en', 'alpha2') == Language('eng'))
        with self.assertRaises(ReverseError):
            Language.fromalpha2('zz')
        with self.assertRaises(ConvertError):
            Language('aaa').alpha2
        self.assertTrue(len(CONVERTERS['alpha2'].codes) == 184)

    def test_converter_alpha3b(self):
        self.assertTrue(Language('fra').alpha3b == 'fre')
        self.assertTrue(Language.fromalpha3b('fre') == Language('fra'))
        self.assertTrue(Language.fromcode('fre', 'alpha3b') == Language('fra'))
        with self.assertRaises(ReverseError):
            Language.fromalpha3b('zzz')
        with self.assertRaises(ConvertError):
            Language('aaa').alpha3b
        self.assertTrue(len(CONVERTERS['alpha3b'].codes) == 418)

    def test_converter_name(self):
        self.assertTrue(Language('eng').name == 'English')
        self.assertTrue(Language.fromname('English') == Language('eng'))
        self.assertTrue(Language.fromcode('English', 'name') == Language('eng'))
        with self.assertRaises(ReverseError):
            Language.fromname('Zzzzzzzzz')
        self.assertTrue(len(CONVERTERS['name'].codes) == 7874)

    def test_converter_opensubtitles(self):
        self.assertTrue(Language('fra').opensubtitles == Language('fra').alpha3b)
        self.assertTrue(Language('por', 'BR').opensubtitles == 'pob')
        self.assertTrue(Language.fromopensubtitles('fre') == Language('fra'))
        self.assertTrue(Language.fromopensubtitles('pob') == Language('por', 'BR'))
        self.assertTrue(Language.fromcode('pob', 'opensubtitles') == Language('por', 'BR'))
        with self.assertRaises(ReverseError):
            Language.fromopensubtitles('zzz')
        with self.assertRaises(ConvertError):
            Language('aaa').opensubtitles
        self.assertTrue(len(CONVERTERS['opensubtitles'].codes) == 419)

    def test_country(self):
        self.assertTrue(Language('por', 'BR').country == Country('BR'))
        self.assertTrue(Language('eng', Country('US')).country == Country('US'))

    def test_eq(self):
        self.assertTrue(Language('eng') == Language('eng'))

    def test_eq_with_country(self):
        self.assertTrue(Language('eng', 'US') == Language('eng', Country('US')))

    def test_ne(self):
        self.assertTrue(Language('fra') != Language('eng'))

    def test_ne_with_country(self):
        self.assertTrue(Language('eng', 'US') != Language('eng', Country('GB')))

    def test_hash(self):
        self.assertTrue(hash(Language('fra')) == hash('fra'))

    def test_register_converter(self):
        class TestConverter(ReverseConverter):
            def __init__(self):
                self.to_test = {'fra': 'test1', 'eng': 'test2'}
                self.from_test = {'test1': 'fra', 'test2': 'eng'}

            def convert(self, alpha3, country=None):
                if alpha3 not in self.to_test:
                    raise ConvertError(alpha3, country)
                return self.to_test[alpha3]

            def reverse(self, test):
                if test not in self.from_test:
                    raise ReverseError(test)
                return (self.from_test[test], None)
        language = Language('fra')
        self.assertTrue(not hasattr(language, 'test'))
        register_converter('test', TestConverter)
        self.assertTrue(hasattr(language, 'test'))
        self.assertTrue('test' in CONVERTERS)
        self.assertTrue(Language('fra').test == 'test1')
        self.assertTrue(Language.fromtest('test2').alpha3 == 'eng')
        unregister_converter('test')
        self.assertTrue('test' not in CONVERTERS)
        self.assertTrue(not hasattr(Language, 'fromtest'))
        with self.assertRaises(AttributeError):
            Language('fra').test
        clear_converters()
        load_converters()


def suite():
    suite = TestSuite()
    suite.addTest(TestLoader().loadTestsFromTestCase(TestCountry))
    suite.addTest(TestLoader().loadTestsFromTestCase(TestLanguage))
    return suite


if __name__ == '__main__':
    TextTestRunner().run(suite())



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from functools import partial
from pkg_resources import resource_stream, iter_entry_points  # @UnresolvedImport
from .converters import ReverseConverter
from .country import Country


CONVERTERS = {}
LANGUAGES = set()
with resource_stream('babelfish', 'data/iso-639-3.tab') as f:
    f.readline()
    for l in f:
        (alpha3, _, _, _, _, _, _, _) = l.decode('utf-8').split('\t')
        LANGUAGES.add(alpha3)


class Language(object):
    """A human language

    A human language is composed of a language part following the ISO-639
    standard and can be country-specific when a :class:`~babelfish.country.Country`
    is specified.

    The :class:`Language` is extensible with custom converters (see :ref:`custom_converters`)

    :param string language: the language as a three-letters ISO-639-3 code
    :param country: the country (if any) as a two-letters ISO-3166 code or :class:`~babelfish.country.Country` instance
    :type country: string or :class:`~babelfish.country.Country` or None

    """
    def __init__(self, language, country=None):
        if language not in LANGUAGES:
            raise ValueError('%r is not a valid language' % language)
        self.alpha3 = language
        self.country = None
        if isinstance(country, Country):
            self.country = country
        elif country is None:
            self.country = None
        else:
            self.country = Country(country)

    @classmethod
    def fromcode(cls, code, converter):
        return cls(*CONVERTERS[converter].reverse(code))

    def __getattr__(self, name):
        if name not in CONVERTERS:
            raise AttributeError
        return CONVERTERS[name].convert(self.alpha3, self.country.alpha2 if self.country is not None else None)

    def __hash__(self):
        if self.country is None:
            return hash(self.alpha3)
        return hash(self.alpha3 + '-' + self.country.alpha2)

    def __eq__(self, other):
        return self.alpha3 == other.alpha3 and self.country == other.country

    def __ne__(self, other):
        return not self == other

    def __repr__(self):
        return '<Language [%s]>' % self

    def __str__(self):
        if self.country is not None:
            return '%s-%s' % (self.alpha3, self.country)
        return self.alpha3


def register_converter(name, converter):
    """Register a :class:`~babelfish.converters.Converter`
    with the given name

    This will add the `name` property to the :class:`Language` class and
    an alternative constructor `fromname` if the `converter` is a
    :class:`~babelfish.converters.ReverseConverter`

    :param string name: name of the converter to register
    :param converter: converter to register
    :type converter: :class:`~babelfish.converters.Converter`

    """
    if name in CONVERTERS:
        raise ValueError('Converter %r already exists' % name)
    CONVERTERS[name] = converter()
    if isinstance(CONVERTERS[name], ReverseConverter):
        setattr(Language, 'from' + name, partial(Language.fromcode, converter=name))


def unregister_converter(name):
    """Unregister a :class:`~babelfish.converters.Converter` by
    name

    :param string name: name of the converter to unregister

    """
    if name not in CONVERTERS:
        raise ValueError('Converter %r does not exist' % name)
    if isinstance(CONVERTERS[name], ReverseConverter):
        delattr(Language, 'from' + name)
    del CONVERTERS[name]


def load_converters():
    """Load converters from the entry point

    Call :func:`register_converter` for each entry of the
    'babelfish.converters' entry point

    """
    for ep in iter_entry_points('babelfish.converters'):
        register_converter(ep.name, ep.load())


def clear_converters():
    """Clear all converters

    Call :func:`unregister_converter` on each registered converter
    in :data:`CONVERTERS`

    """
    for name in set(CONVERTERS.keys()):
        unregister_converter(name)
